@@ -1156,7 +1156,7 @@ void php_mysqli_fetch_into_hash_aux(zval *return_value, MYSQL_RES * result, zend
 {
 #if !defined(MYSQLI_USE_MYSQLND)
 	MYSQL_ROW row;
-	unsigned int	i, num_fields;
+	unsigned int	i;
 	MYSQL_FIELD		*fields;
 	zend_ulong	*field_len;
 
@@ -1170,9 +1170,8 @@ void php_mysqli_fetch_into_hash_aux(zval *return_value, MYSQL_RES * result, zend
 
 	array_init(return_value);
 	field_len = mysql_fetch_lengths(result);
-	num_fields = mysql_num_fields(result);
 
-	for (i = 0; i < num_fields; i++) {
+	for (i = 0; i < mysql_num_fields(result); i++) {
 		if (row[i]) {
 			zval res;
 
@@ -1252,10 +1251,6 @@ void php_mysqli_fetch_into_hash(INTERNAL_FUNCTION_PARAMETERS, int override_flags
 			php_error_docref(NULL, E_WARNING, "Could not find class '%s'", ZSTR_VAL(class_name));
 			return;
 		}
-		if (UNEXPECTED(ce->ce_flags & (ZEND_ACC_INTERFACE|ZEND_ACC_TRAIT|ZEND_ACC_IMPLICIT_ABSTRACT_CLASS|ZEND_ACC_EXPLICIT_ABSTRACT_CLASS))) {
-			zend_throw_error(NULL, "Class '%s' cannot be instantiated", ZSTR_VAL(ce->name));
-			return;
-		}
 		fetchtype = MYSQLI_ASSOC;
 	} else {
 		if (override_flags) {
